// Copyright (c) 2015, Electric Cloud, Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   - Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//
//   - Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// (As an historical note, the above license instantiates
// the "BSD 2-Clause License" template that was presented
// by the Open Source Initiative at
//
//   http://www.opensource.org/licenses/bsd-license.php
//
// on 2012-02-23.)

// Summary:
//
// We do not know where exactly "move" will show up in
// the Boost libraries.  This header gets Boost "move"
// into the "vrpsx" namespace from wherever it lives.

#ifndef VRPSX_MOVE_HPP
#define VRPSX_MOVE_HPP 1

#if defined(_MSC_VER)
#pragma warning ( push )
#pragma warning ( disable : 4946 )
#endif

// Ensure that boost::move or boost::interprocess::move is declared:

#include <boost/interprocess/smart_ptr/unique_ptr.hpp>

#if defined(_MSC_VER)
#pragma warning ( pop )
#endif


namespace boost {
    // Ensure that "using boost::move;" will succeed,
    // but without actually defining anything we might actually use:

    template< typename Unused > void move(int, int, int, int, int, int);

    namespace interprocess {
        // Ensure that "using boost::interprocess::move;" will succeed,
        // but without actually defining anything we might actually use:

        template< typename Unused > void move(int, int, int, int, int, int);
    }
}

namespace vrpsx {
    // Include all the possible declarations of Boost "move":

    using boost::move;
    using boost::interprocess::move;
}


#endif /* VRPSX_MOVE_HPP */
