/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * This is an example program that makes use of a virtual POSIX-like interface.
 *
 * This program prints out information about the current mounts.
 */

#define __STRICT_ANSI__ 1

#include <vrpsx_system_main.h>
#include <vrpsx_system_main_identifier.h>
#include <vrpsx_identified_function.h>
#include <vrpsx_identified_function_identifier.h>
#include <vrpsx_utility.h>
#include <string.h>


VRPSX_DECLARE_IDENTIFIED_FUNCTION(static, main, vrpsx_system_main);

static int VRPSX_IDENTIFIED_FUNCTION_NAME(main, vrpsx_system_main)(
    vrpsx_system_initial_t *initial)
{
    vrpsx_system_t *system = initial->system;
    vrpsx_mount_t *mount;
    vrpsx_substring_t device, root, type;
    vrpsx_mount_option_t *option;
    vrpsx_system_error_t *error;
    char const *base;
    char const *operation = 0;
    vrpsx_errno_t ern = 0;

    ern = VRPSX_CALL1(system, get_mounts, &mount);
    if (ern) {
        operation = "get_mounts";
        goto cleanup;
    }

    while (mount) {
        ern = VRPSX_CALL1(mount, device, &device);
        if (ern) {
            operation = "vrpsx_mount_t::device";
            goto cleanup;
        }
        ern = VRPSX_CALL1(mount, root, &root);
        if (ern) {
            operation = "vrpsx_mount_t::root";
            goto cleanup;
        }
        ern = VRPSX_CALL1(mount, type, &type);
        if (ern) {
            operation = "vrpsx_mount_t::type";
            goto cleanup;
        }

        ern = VRPSX_CALL4(system, write, 1, "device: ", 8, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }
        ern = VRPSX_CALL4(system, write, 1, device.begin, device.size, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }
        ern = VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }

        ern = VRPSX_CALL4(system, write, 1, "root: ", 6, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }
        ern = VRPSX_CALL4(system, write, 1, root.begin, root.size, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }
        ern = VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }

        ern = VRPSX_CALL4(system, write, 1, "type: ", 6, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }
        ern = VRPSX_CALL4(system, write, 1, type.begin, type.size, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }
        ern = VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }

        ern = VRPSX_CALL1(mount, options, &option);
        if (ern) {
            operation = "vrpsx_mount_t::options";
            goto cleanup;
        }

        while (option) {
            vrpsx_substring_t text;
            ern = VRPSX_CALL1(option, text, &text);
            if (ern) {
                operation = "vrpsx_mount_option_t::text";
                goto cleanup;
            }

            ern = VRPSX_CALL4(system, write, 1, "option: ", 8, NULL);
            if (ern) {
                operation = "write";
                goto cleanup;
            }
            ern = VRPSX_CALL4(system, write, 1, text.begin, text.size,NULL);
            if (ern) {
                operation = "write";
                goto cleanup;
            }
            ern = VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
            if (ern) {
                operation = "write";
                goto cleanup;
            }

            VRPSX_SINGLE_OWNER_LIST_NEXT_CHECK_SROE(
                vrpsx_mount_option_t, &option, &option, &ern, NULL);
            if (ern) {
                operation = "vrpsx_mount_option_t::next";
                goto cleanup;
            }
        }

        ern = VRPSX_CALL4(system, write, 1, "\n", 1, NULL);
        if (ern) {
            operation = "write";
            goto cleanup;
        }

        VRPSX_SINGLE_OWNER_LIST_NEXT_CHECK_SROE(
            vrpsx_mount_t, &mount, &mount, &ern, NULL);
        if (ern) {
            operation = "vrpsx_mount_t::next";
            goto cleanup;
        }
    }

cleanup:
    if (ern) {
        VRPSX_CALL2(system, error_acquire, ern, &error);
        base = VRPSX_CALL0(error, text);

        VRPSX_CALL4(system, write, 2, operation, strlen(operation), NULL);
        VRPSX_CALL4(system, write, 2, ": ", 2, NULL);
        VRPSX_CALL4(system, write, 2, base, strlen(base), NULL);

        VRPSX_COUNTED_REFERENCE_CLEAR(vrpsx_system_error_t, &error);

        return VRPSX_CONSTANT(system, exit_failure);
    }

    return VRPSX_CONSTANT(system, exit_success);
}

vrpsx_identified_function_t const VRPSX_IDENTIFIED_FUNCTION_LIST[] = {
    VRPSX_MENTION_IDENTIFIED_FUNCTION(main, vrpsx_system_main),
    VRPSX_IDENTIFIED_FUNCTION_LIST_TERMINATOR
};
