/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * Implements default methods for the vrpsx_ostreambuf_t interface.
 */

#define _POSIX_C_SOURCE 200809L

#include "vrpsx_ostreambuf_default.h"
#include "vrpsx_utility.h"
#include <assert.h>
#include <string.h>


int vrpsx_ostreambuf_put1_from_flush(
    vrpsx_ostreambuf_t *osb,
    char ch)
{
    int ern;

    if (osb->error) {
        return osb->error;
    }

    if (osb->current == osb->end) {
        ern = VRPSX_CALL0(osb, flush);
        if (ern) {
            return ern;
        }
        assert (osb->current != osb->end);
    }

    *osb->current++ = ch;
    return 0;
}

int vrpsx_ostreambuf_putn_from_put1(
    vrpsx_ostreambuf_t *osb,
    char const *begin,
    size_t size)
{
    size_t block;
    int ern;

    if (osb->error) {
        return osb->error;
    }

    while (size) {
        block = osb->end - osb->current;
        block = block < size ? block : size;

        if (block) {
            memcpy(osb->current, begin, block);
            osb->current += block;
            begin += block;
            size -= block;
        } else {
            ern = VRPSX_CALL1(osb, put1, *begin);
            if (ern) {
                return ern;
            }
            ++ begin;
            -- size;
        }
    }

    return 0;
}
